bugfix - add stl-model.h wrappers, to provide more control over STL
[c11tester.git] / promise.h
index 278b7cc224e34a262ebbca9c20d3294ede514d31..90ec1d2050bc650b714eebe8f64f5aabde0de7e0 100644 (file)
--- a/promise.h
+++ b/promise.h
@@ -8,10 +8,10 @@
 #define __PROMISE_H__
 
 #include <inttypes.h>
-#include <vector>
 
 #include "modeltypes.h"
 #include "mymemory.h"
+#include "stl-model.h"
 
 class ModelAction;
 
@@ -24,34 +24,42 @@ struct future_value {
 class Promise {
  public:
        Promise(ModelAction *read, struct future_value fv);
-       modelclock_t get_expiration() const { return expiration; }
-       ModelAction * get_action() const { return read; }
+       bool add_reader(ModelAction *reader);
+       ModelAction * get_reader(unsigned int i) const;
+       unsigned int get_num_readers() const { return readers.size(); }
        bool eliminate_thread(thread_id_t tid);
        void add_thread(thread_id_t tid);
        bool thread_is_available(thread_id_t tid) const;
        bool has_failed() const;
-       uint64_t get_value() const { return value; }
        void set_write(const ModelAction *act) { write = act; }
        const ModelAction * get_write() const { return write; }
        int get_num_available_threads() const { return num_available_threads; }
        bool is_compatible(const ModelAction *act) const;
        bool is_compatible_exclusive(const ModelAction *act) const;
+       bool same_value(const ModelAction *write) const;
+       bool same_location(const ModelAction *act) const;
+
+       modelclock_t get_expiration() const { return fv.expiration; }
+       uint64_t get_value() const { return fv.value; }
+       struct future_value get_fv() const { return fv; }
 
        void print() const;
 
+       bool equals(const Promise *x) const { return this == x; }
+       bool equals(const ModelAction *x) const { return false; }
+
        SNAPSHOTALLOC
  private:
        /** @brief Thread ID(s) for thread(s) that potentially can satisfy this
         *  promise */
-       std::vector< bool, SnapshotAlloc<bool> > available_thread;
+       SnapVector<bool> available_thread;
 
        int num_available_threads;
 
-       const uint64_t value;
-       const modelclock_t expiration;
+       const future_value fv;
 
-       /** @brief The action which reads a promised value */
-       ModelAction * const read;
+       /** @brief The action(s) which read the promised future value */
+       SnapVector<ModelAction *> readers;
 
        const ModelAction *write;
 };